@@ -12,7 +12,8 @@ from pyjoystick.qt.widgets import AxisWidget, ButtonWidget, HatWidget, SpinSlide
 from pyjoystick.qt.keys_trigger import RESOURCE_DIR
 
 
-__all__ = ["JOYSTICK_ACTIVE_IMG", "JOYSTICK_INACTIVE_IMG", "JoystickKeyMapper"]
+__all__ = ["JOYSTICK_ACTIVE_IMG", "JOYSTICK_INACTIVE_IMG",
+           'JoystickKeyMapperMixin', 'JoystickKeyMapper', 'JoystickKeyMapperDialog']
 
 
 # ========== ICONS ==========
@@ -21,19 +22,53 @@ JOYSTICK_INACTIVE_IMG = os.path.join(RESOURCE_DIR, "gamepad_inactive.png")
 # ========== END ICONS ==========
 
 
-class JoystickKeyMapper(QtWidgets.QWidget):
+class JoystickKeyMapperMixin(object):
     """Help the user map the keys."""
 
     Joystick = Joystick
 
-    def __init__(self, joystick=None, parent=None, event_mngr=None):
-        super().__init__(parent)
+    def __init__(self, joystick=None, event_mngr=None, **kwargs):
+        super().__init__()
 
         self._event_mngr = None
         self._joystick = None
         self._joysticks = []
         self.btns_per_row = 5
+        self._lock = None
 
+        # Qt variables
+        self.main_layout = None
+        self.name_lay = None
+        self.name_lbl = None
+        self.deadband_layout = None
+        self.deadband_input = None
+        self.axis_layout = None
+        self.button_layout = None
+        self.hat_layout = None
+        self.ball_layout = None
+        self.sel_btn = None
+        self._dialog = None
+
+        self.init(joystick=joystick, event_mngr=event_mngr)
+
+    def init(self, joystick=None, event_mngr=None):
+        self._event_mngr = None
+        self._joystick = None
+        self._joysticks = []
+        self.btns_per_row = 5
+
+        self.init_layout()
+
+        # Items
+        self._lock = threading.Lock()
+        if joystick is not None:
+            self.sel_btn.hide()
+            self.set_joystick(joystick)
+
+        if event_mngr is not None:
+            self.set_event_mngr(event_mngr)
+
+    def init_layout(self):
         # Layout
         self.main_layout = QtWidgets.QVBoxLayout()
         self.setLayout(self.main_layout)
@@ -48,6 +83,18 @@ class JoystickKeyMapper(QtWidgets.QWidget):
         self.name_lay = QtWidgets.QHBoxLayout()
         self.name_lay.addWidget(self.name_lbl, alignment=QtCore.Qt.AlignLeft)
         self.main_layout.addLayout(self.name_lay)
+
+        self.deadband_input = QtWidgets.QDoubleSpinBox()
+        self.deadband_input.setToolTip('Give the joystick some dead space for the analog sticks.\n'
+                                       'This helps analog sticks rest at 0.')
+        self.deadband_input.setMinimum(0)
+        self.deadband_input.setMaximum(1)
+        self.deadband_input.setDecimals(4)
+        self.deadband_input.valueChanged.connect(self.set_deadband)
+        self.deadband_layout = QtWidgets.QFormLayout()
+        self.deadband_layout.addRow(QtWidgets.QLabel('Deadband: '), self.deadband_input)
+        self.main_layout.addLayout(self.deadband_layout)
+
         
         # Button layouts
         lbl = QtWidgets.QLabel("Axes (Analog sticks or triggers):")
@@ -76,16 +123,6 @@ class JoystickKeyMapper(QtWidgets.QWidget):
         self.name_lay.addWidget(self.sel_btn, alignment=QtCore.Qt.AlignLeft)
         self._dialog = None
 
-        # Items
-        self._lock = threading.Lock()
-        if joystick is not None:
-            self.sel_btn.hide()
-            self.set_joystick(joystick)
-
-        if event_mngr is not None:
-            self.set_event_mngr(event_mngr)
-    # end Constructor
-
     def get_event_mngr(self):
         """Return the event manager."""
         return self._event_mngr
@@ -205,6 +242,8 @@ class JoystickKeyMapper(QtWidgets.QWidget):
         """Initialize the widgets."""
         self.removeWidgets()
 
+        self.deadband_input.setValue(joystick.deadband)
+
         for i in range(joystick.get_numaxes()):
             widg = AxisWidget("Axis "+str(i))
             self.axis_layout.addWidget(widg)
@@ -235,6 +274,15 @@ class JoystickKeyMapper(QtWidgets.QWidget):
             self.removeWidgets()
             return
 
+        # Try to get the proper joystick for this event loop
+        try:
+            event_mngr = self.get_event_mngr()
+            if joystick not in event_mngr.joysticks:
+                event_mngr.save_joystick(joystick)
+            joystick = event_mngr.joysticks[joystick]
+        except (AttributeError, IndexError, KeyError, Exception):
+            pass
+
         self._joystick = joystick
         self.createWidgets(self._joystick)
         try:
@@ -244,6 +292,12 @@ class JoystickKeyMapper(QtWidgets.QWidget):
 
     joystick = property(get_joystick, set_joystick)
 
+    def set_deadband(self, value):
+        try:
+            self.joystick.set_deadband(value)
+        except:
+            pass
+
     def set_axes_values(self, key):
         try:
             self.axis_layout.itemAt(key.number).widget().setValue(key.get_value())
@@ -277,6 +331,20 @@ class JoystickKeyMapper(QtWidgets.QWidget):
         return QtWidgets.QWidget.closeEvent(self, *args, **kwargs)
 
 
+class JoystickKeyMapper(QtWidgets.QWidget, JoystickKeyMapperMixin):
+    """Help the user map the keys."""
+    def __init__(self, joystick=None, event_mngr=None, parent=None):
+        QtWidgets.QWidget.__init__(self, parent=parent)
+        self.init(joystick=joystick, event_mngr=event_mngr)
+
+
+class JoystickKeyMapperDialog(QtWidgets.QDialog, JoystickKeyMapperMixin):
+    """Help the user map the keys."""
+    def __init__(self, joystick=None, event_mngr=None, parent=None):
+        QtWidgets.QDialog.__init__(self, parent=parent)
+        self.init(joystick=joystick, event_mngr=event_mngr)
+
+
 if __name__ == "__main__":
     import sys
     from pyjoystick import ThreadEventManager, ButtonHatRepeater
@@ -286,7 +354,7 @@ if __name__ == "__main__":
 
     app = QtWidgets.QApplication([])
 
-    w = JoystickKeyMapper(event_mngr=ThreadEventManager(run_event_loop, button_repeater=ButtonHatRepeater()))  # , activity_timeout=0.02))
+    w = JoystickKeyMapperDialog(event_mngr=ThreadEventManager(run_event_loop, button_repeater=ButtonHatRepeater()))  # , activity_timeout=0.02))
     # w = JoystickKeyMapper(event_mngr=MultiprocessingEventManager(run_event_loop, button_repeater=ButtonHatRepeater()))  # , activity_timeout=0.02))
     w.show()
 
