@@ -1,11 +1,12 @@
 import ctypes
+import asyncio
 
 # ========== SDL2 Pathing ==========
 import os
 import sys
 import platform
 
-from pyjoystick.utils import PYJOYSTICK_DIR, deadband, change_path, rescale
+from pyjoystick.utils import PYJOYSTICK_DIR, change_path, rescale
 
 # ========== SDL2 Pathing ==========
 if 'windows' in platform.architecture()[1].lower():
@@ -22,7 +23,7 @@ from pyjoystick.stash import Stash
 from pyjoystick.interface import Key, Joystick as BaseJoystick
 
 
-__all__ = ['Key', 'Joystick', 'run_event_loop',
+__all__ = ['Key', 'Joystick', 'run_event_loop', 'stop_event_wait',
            'get_init', 'init', 'quit', 'key_from_event',
            'get_guid', 'get_mapping', 'get_mapping_name', 'is_trigger']
 
@@ -96,7 +97,7 @@ class Joystick(BaseJoystick):
 
         return joy
 
-    def is_active(self):
+    def is_available(self):
         """Return if this joystick is still active and available."""
         try:
             return sdl2.SDL_JoystickGetAttached(self.joystick)
@@ -304,11 +305,6 @@ def key_from_event(event, joystick=None):
             value = rescale(value, -32768, 32767, 0, 1)  # Make triggers rest at 0
         else:
             value = rescale(value, -32768, 32767, -1, 1)
-
-        db = getattr(joystick, 'deadband', 0)
-        if db:
-            value = deadband(value, db)
-
         return Key(Key.AXIS, event.jaxis.axis, value, joystick)
     elif event.type == sdl2.SDL_JOYHATMOTION:
         return Key(Key.HAT, event.jhat.hat, event.jhat.value, joystick)
@@ -318,6 +314,19 @@ def key_from_event(event, joystick=None):
     return None
 
 
+def stop_event_wait():
+    """Post an event to break out of the event loop wait."""
+    try:
+        user_event = sdl2.SDL_Event()
+        user_event.type = sdl2.SDL_USEREVENT
+        user_event.user.code = 2
+        user_event.user.data1 = None
+        user_event.user.data2 = None
+        sdl2.SDL_PushEvent(ctypes.byref(user_event))
+    except:
+        pass
+
+
 def run_event_loop(add_joystick, remove_joystick, handle_key_event, alive=None, **kwargs):
     """Run the an event loop to process SDL Events.
 
@@ -335,7 +344,7 @@ def run_event_loop(add_joystick, remove_joystick, handle_key_event, alive=None,
 
     event = sdl2.SDL_Event()
     while alive():
-        if sdl2.SDL_WaitEvent(ctypes.byref(event)) != 0:
+        if sdl2.SDL_WaitEventTimeout(ctypes.byref(event), 2000) != 0:
             # Check the event
             if event.type == sdl2.SDL_JOYDEVICEADDED:
                 try:
@@ -357,3 +366,74 @@ def run_event_loop(add_joystick, remove_joystick, handle_key_event, alive=None,
                 key = key_from_event(event, joy)
                 if key is not None:
                     handle_key_event(key)
+
+
+# Attach a way to stop waiting by posting an event
+run_event_loop.stop_event_wait = stop_event_wait
+
+
+async def get_async_event(event: sdl2.SDL_Event = None, alive=None) -> sdl2.SDL_Event:
+    if alive is None:
+        alive = lambda: True
+    if event is None:
+        event = sdl2.SDL_Event()
+
+    # while True:
+    #     if sdl2.SDL_PollEvent(ctypes.byref(event)):
+    #         return
+    #     else:
+    #         await asyncio.sleep(0)
+
+    loop = asyncio.get_running_loop()
+    while alive():
+        if await loop.run_in_executor(None, sdl2.SDL_WaitEventTimeout, ctypes.byref(event), 2000) != 0:
+            return event
+    return None
+
+
+async def async_event_loop(add_joystick, remove_joystick, handle_key_event, alive=None, **kwargs):
+    """Run the an event loop to process SDL Events.
+
+    Args:
+        add_joystick (callable/function): Called when a new Joystick is found!
+        remove_joystick (callable/function): Called when a Joystick is removed!
+        handle_key_event (callable/function): Called when a new key event occurs!
+        alive (callable/function)[None]: Function to return True to continue running. If None run forever
+    """
+    if alive is None:
+        alive = lambda: True
+
+    if not get_init():
+        init()
+
+    event = sdl2.SDL_Event()
+    while alive():
+        result = await get_async_event(event, alive=alive)
+        if result is None:
+            break
+
+        # Check the event
+        if event.type == sdl2.SDL_JOYDEVICEADDED:
+            try:
+                # NOTE: event.jdevice.which is the id to use for SDL_JoystickOpen()
+                joy = Joystick(identifier=event.jdevice.which)
+                await add_joystick(joy)
+            except:
+                pass
+        elif event.type == sdl2.SDL_JOYDEVICEREMOVED:
+            try:
+                # NOTE: event.jdevice.which is the id to use for SDL_JoystickFromInstanceID()
+                joy = Joystick(instance_id=event.jdevice.which)
+                await remove_joystick(joy)
+            except:
+                pass
+        else:
+            # NOTE: event.jdevice.which is the id to use for SDL_JoystickFromInstanceID()
+            joy = Joystick(instance_id=event.jdevice.which)
+            key = key_from_event(event, joy)
+            if key is not None:
+                await handle_key_event(key)
+
+
+# Attach a way to stop waiting by posting an event
+async_event_loop.stop_event_wait = stop_event_wait
